# This file specifies the marker genes methods (and their parameters) which
# will be assessed.

# Read the overall config file.
config <- yaml::read_yaml(here::here("config.yaml"))

# The methods to be run.
# FIXME: Currently UNUSED.
methods <- c("scran", "seurat")

scran <- expand.grid(method = "scran",
                     test.type = c("t", "wilcox"),
                     pval.type = c("any", "all"))

seurat <- expand.grid(method = "seurat",
                      test.use = c("wilcox", "bimod", "roc", "t", "negbinom",
                                   "poisson", "LR", "MAST", "DESeq2"))


method_pars <- list(scran, seurat)

# Convert each row of each set of parameters into a string describing the
# simulations.
method_ids <- lapply(method_pars, function(x) {
  apply(x, 1, function(x) paste0(x, collapse = "_"))
})
method_ids <- unlist(method_ids)

# We want each row to be a list.
method_pars <- lapply(method_pars, function(x) {
  apply(x, 1, function(x) as.list(x))
})
# Remove the top layer (overall method) from the list of lists.
method_pars <- unlist(method_pars, recursive = FALSE)

# Add the id to each list of parameters.
names(method_pars) <- method_ids

# TODO: Use JSON or a csv?

# Write out the method ids to a (JSON) file.
write(jsonlite::toJSON(method_ids), config$method_ids_path)

# FIXME
# Construct the paths of the method id files.
# dir.create(config$method_pars_folder, showWarnings = FALSE)

method_paths <- here::here(config$method_pars_folder, method_ids)
method_paths <- paste0(method_paths, ".json")
names(method_paths) <- method_ids

# Write all the parameters of each method to file.
# The checking of previous parameters is needed so that we don't constantly
# make the files newer, breaking Snakemake.
# FIXME: How can we clean up out of date parameter files?
for (id in method_ids) {
  new <- method_pars[[id]]
  path <- method_paths[[id]]

  if (file.exists(path)) {
    old <- jsonlite::fromJSON(path)
    if (!identical(old, new)) {
      # The file already exists, but we changed it.
      write(jsonlite::toJSON(new, null = "null"), file)
    }
  } else {
    # This file doesn't already exist.
    write(jsonlite::toJSON(new, null = "null"), file)
  }
}
